import json, requests

class APITestController():
    def __init__(self, config_file_path):
        self.configuration = json.loads(open(config_file_path, "r").read())

    @property
    def start(self):
        for request in self.configuration:
            self.perform_test(request)
        return True

    def perform_test(self, request_info):
        print("\n\nPerforming Test\n")
        params = {}
        body = {}

        if "params" in request_info:
            params = request_info["params"]
        if "body" in request_info:
            body = request_info["body"]
        resp, success = self.perform_request(
            request_info["type"],
            request_info["url"],
            request_info["headers"],
            body,
            params,
        )

        if not success:
            print ("There was some error with the api\n\n")
        else:
            print("API Call was successful\n\n")

    def recreate_url(self, url, params):
        split_keyword = "?"
        for key in params:
            url += split_keyword + key + "=" + params[key]
            split_keyword = "&"
        return url

    def perform_request(self, type, url, headers, jsonBody=None, param=None):
        request_controller = None
        if type == "GET":
            request_controller = requests.get
        elif type == "POST":
            request_controller = requests.post
        elif type == "PUT":
            request_controller = requests.put
        elif type == "DELETE":
            request_controller = requests.delete
        else:
            raise ValueError("Unknown URL type for API Test. "
                             "Supported ( GET, POST, PUT, DELET ) methods only")

        if param != None:
            url = self.recreate_url(url, param)


        print("URL :", url)
        print("headers :", headers)
        print("Json Body :", jsonBody)
        print("Type :", type)
        resp = request_controller(
            url=url,
            headers=headers,
            json=jsonBody
        )
        print("Response Code :", resp.status_code)
        print("Response Text :", resp.text)
        success_codes = [
            200, 201, 202
        ]
        if resp.status_code in success_codes:
            return resp.json(), True

        return resp.text, False


if __name__ == '__main__':
    a = APITestController("/Users/jennie/Desktop/ASK_Jennie_automations/jennie/apitestcontroller/config.json").start
